@@ -21,8 +21,7 @@ import sys
 import textwrap
 
 # Third party imports
-import qdarkstyle
-from qtpy.QtCore import QPoint, Qt
+from qtpy.QtCore import QPoint, QRegularExpression, Qt
 from qtpy.QtGui import QCursor, QTextCursor, QTextDocument
 from qtpy.QtWidgets import QApplication
 from qtpy import QT_VERSION
@@ -30,28 +29,21 @@ from spyder_kernels.utils.dochelpers import (getargspecfromtext, getobj,
                                              getsignaturefromtext)
 
 # Local imports
-from spyder.config.gui import is_dark_interface
 from spyder.config.manager import CONF
 from spyder.py3compat import is_text_string, to_text_string
-from spyder.utils import encoding, sourcecode, programs
+from spyder.utils import encoding, sourcecode
 from spyder.utils import syntaxhighlighters as sh
 from spyder.utils.misc import get_error_match
+from spyder.utils.palette import QStylePalette
 from spyder.widgets.arraybuilder import ArrayBuilderDialog
 
-QT55_VERSION = programs.check_version(QT_VERSION, "5.5", ">=")
-
-if QT55_VERSION:
-    from qtpy.QtCore import QRegularExpression
-else:
-    from qtpy.QtCore import QRegExp
-
 
 class BaseEditMixin(object):
 
-    _PARAMETER_HIGHLIGHT_COLOR = '#DAA520'
-    _DEFAULT_TITLE_COLOR = '#2D62FF'
-    _CHAR_HIGHLIGHT_COLOR = 'red'
-    _DEFAULT_TEXT_COLOR = '#999999'
+    _PARAMETER_HIGHLIGHT_COLOR = QStylePalette.COLOR_ACCENT_4
+    _DEFAULT_TITLE_COLOR = QStylePalette.COLOR_ACCENT_4
+    _CHAR_HIGHLIGHT_COLOR = QStylePalette.COLOR_ACCENT_4
+    _DEFAULT_TEXT_COLOR = QStylePalette.COLOR_TEXT_2
     _DEFAULT_LANGUAGE = 'python'
     _DEFAULT_MAX_LINES = 10
     _DEFAULT_MAX_WIDTH = 60
@@ -134,21 +126,16 @@ class BaseEditMixin(object):
         if id(widget) in self._styled_widgets:
             return
         self._styled_widgets.add(id(widget))
-
-        if is_dark_interface():
-            css = qdarkstyle.load_stylesheet(qt_api='')
-            widget.setStyleSheet(css)
-            palette = widget.palette()
-            background = palette.color(palette.Window).lighter(150).name()
-            border = palette.color(palette.Window).lighter(200).name()
-            name = widget.__class__.__name__
-            widget.setObjectName(name)
-            extra_css = '''
-                {0}#{0} {{
-                    background-color:{1};
-                    border: 1px solid {2};
-                }}'''.format(name, background, border)
-            widget.setStyleSheet(css + extra_css)
+        background = QStylePalette.COLOR_BACKGROUND_4
+        border = QStylePalette.COLOR_TEXT_4
+        name = widget.__class__.__name__
+        widget.setObjectName(name)
+        css = '''
+            {0}#{0} {{
+                background-color:{1};
+                border: 1px solid {2};
+            }}'''.format(name, background, border)
+        widget.setStyleSheet(css)
 
     def _get_inspect_shortcut(self):
         """
@@ -299,7 +286,8 @@ class BaseEditMixin(object):
                 shortcut = self._get_inspect_shortcut()
                 if shortcut:
                     base_style = (
-                        'background-color:#fafbfc;color:#444d56;'
+                        f'background-color:{QStylePalette.COLOR_BACKGROUND_4};'
+                        f'color:{QStylePalette.COLOR_TEXT_1};'
                         'font-size:11px;'
                     )
                     help_text = ''
@@ -316,7 +304,8 @@ class BaseEditMixin(object):
                 template += (
                     '<hr>'
                     '<div align="left">'
-                    '<span style="color:#148CD2;text-decoration:none;'
+                    f'<span style="color: {QStylePalette.COLOR_ACCENT_4};'
+                    'text-decoration:none;'
                     'font-family:"{font_family}";font-size:{size}pt;><i>'
                     ''.format(font_family=font_family,
                               size=text_size)
@@ -696,15 +685,17 @@ class BaseEditMixin(object):
             return os.linesep
 
     def get_text_with_eol(self):
-        """Same as 'toPlainText', replace '\n'
-        by correct end-of-line characters"""
-        utext = to_text_string(self.toPlainText())
-        lines = utext.splitlines()
+        """
+        Same as 'toPlainText', replacing '\n' by correct end-of-line
+        characters.
+        """
+        text = self.toPlainText()
+        lines = text.splitlines()
         linesep = self.get_line_separator()
-        txt = linesep.join(lines)
-        if utext.endswith('\n'):
-            txt += linesep
-        return txt
+        text_with_eol = linesep.join(lines)
+        if text.endswith('\n'):
+            text_with_eol += linesep
+        return text_with_eol
 
     #------Positions, coordinates (cursor, EOF, ...)
     def get_position(self, subject):
@@ -853,12 +844,6 @@ class BaseEditMixin(object):
 
 
     #------Selection
-    def clear_selection(self):
-        """Clear current selection"""
-        cursor = self.textCursor()
-        cursor.clearSelection()
-        self.setTextCursor(cursor)
-
     def extend_selection_to_next(self, what='word', direction='left'):
         """
         Extend selection to next *what* ('word' or 'character')
@@ -980,7 +965,7 @@ class BaseEditMixin(object):
         current_word = self.get_current_word(help_req=True)
 
         # Get max position to the left of cursor until space or no more
-        # charaters are left
+        # characters are left
         cursor.movePosition(QTextCursor.PreviousCharacter)
         while self.get_character(cursor.position()).strip():
             cursor.movePosition(QTextCursor.PreviousCharacter)
@@ -989,7 +974,7 @@ class BaseEditMixin(object):
         cursor_pos_left = cursor.position()
 
         # Get max position to the right of cursor until space or no more
-        # charaters are left
+        # characters are left
         cursor.setPosition(cursor_pos)
         while self.get_character(cursor.position()).strip():
             cursor.movePosition(QTextCursor.NextCharacter)
@@ -1128,11 +1113,15 @@ class BaseEditMixin(object):
 
         return word
 
+    def get_line_indentation(self, text):
+        """Get indentation for given line."""
+        text = text.replace("\t", " "*self.tab_stop_width_spaces)
+        return len(text)-len(text.lstrip())
+
     def get_block_indentation(self, block_nb):
         """Return line indentation (character number)."""
         text = to_text_string(self.document().findBlockByNumber(block_nb).text())
-        text = text.replace("\t", " "*self.tab_stop_width_spaces)
-        return len(text)-len(text.lstrip())
+        return self.get_line_indentation(text)
 
     def get_selection_bounds(self, cursor=None):
         """Return selection bounds (block numbers)."""
@@ -1242,10 +1231,16 @@ class BaseEditMixin(object):
 
     def find_text(self, text, changed=True, forward=True, case=False,
                   word=False, regexp=False):
-        """Find text"""
+        """Find text."""
         cursor = self.textCursor()
         findflag = QTextDocument.FindFlag()
 
+        # Get visible region to center cursor in case it's necessary.
+        if getattr(self, 'get_visible_block_numbers', False):
+            current_visible_region = self.get_visible_block_numbers()
+        else:
+            current_visible_region = None
+
         if not forward:
             findflag = findflag | QTextDocument.FindBackward
 
@@ -1270,16 +1265,10 @@ class BaseEditMixin(object):
         else:
             text = re.escape(to_text_string(text))
 
-        if QT55_VERSION:
-            pattern = QRegularExpression(u"\\b{}\\b".format(text) if word else
-                                         text)
-            if case:
-                pattern.setPatternOptions(
-                    QRegularExpression.CaseInsensitiveOption)
-        else:
-            pattern = QRegExp(u"\\b{}\\b".format(text)
-                              if word else text, Qt.CaseSensitive if case else
-                              Qt.CaseInsensitive, QRegExp.RegExp2)
+        pattern = QRegularExpression(u"\\b{}\\b".format(text) if word else
+                                     text)
+        if case:
+            pattern.setPatternOptions(QRegularExpression.CaseInsensitiveOption)
 
         for move in moves:
             cursor.movePosition(move)
@@ -1293,6 +1282,14 @@ class BaseEditMixin(object):
                 found_cursor = self.document().find(pattern, cursor, findflag)
             if found_cursor is not None and not found_cursor.isNull():
                 self.setTextCursor(found_cursor)
+
+                # Center cursor if we move out of the visible region.
+                if current_visible_region is not None:
+                    found_visible_region = self.get_visible_block_numbers()
+                    if current_visible_region != found_visible_region:
+                        current_visible_region = found_visible_region
+                        self.centerCursor()
+
                 return True
 
         return False
@@ -1389,7 +1386,10 @@ class BaseEditMixin(object):
 class TracebackLinksMixin(object):
     """ """
     QT_CLASS = None
-    go_to_error = None
+
+    # This signal emits a parsed error traceback text so we can then
+    # request opening the file that traceback comes from in the Editor.
+    sig_go_to_error_requested = None
 
     def __init__(self):
         self.__cursor_changed = False
@@ -1401,8 +1401,8 @@ class TracebackLinksMixin(object):
         self.QT_CLASS.mouseReleaseEvent(self, event)
         text = self.get_line_at(event.pos())
         if get_error_match(text) and not self.has_selected_text():
-            if self.go_to_error is not None:
-                self.go_to_error.emit(text)
+            if self.sig_go_to_error_requested is not None:
+                self.sig_go_to_error_requested.emit(text)
 
     def mouseMoveEvent(self, event):
         """Show Pointing Hand Cursor on error messages"""
@@ -1427,13 +1427,9 @@ class TracebackLinksMixin(object):
 
 
 class GetHelpMixin(object):
-    def __init__(self):
-        self.help = None
-        self.help_enabled = False
 
-    def set_help(self, help_plugin):
-        """Set Help DockWidget reference"""
-        self.help = help_plugin
+    def __init__(self):
+        self.help_enabled = False
 
     def set_help_enabled(self, state):
         self.help_enabled = state
@@ -1449,21 +1445,12 @@ class GetHelpMixin(object):
 
         # Show docstring
         help_enabled = self.help_enabled or force
-        if force and self.help is not None:
-            self.help.dockwidget.setVisible(True)
-            self.help.dockwidget.raise_()
-        if help_enabled and (self.help is not None) and \
-           (self.help.dockwidget.isVisible()):
-            # Help widget exists and is visible
-            if hasattr(self, 'get_doc'):
-                self.help.set_shell(self)
-            else:
-                self.help.set_shell(self.parent())
-            self.help.set_object_text(text, ignore_unknown=False)
-            self.setFocus() # if help was not at top level, raising it to
-                            # top will automatically give it focus because of
-                            # the visibility_changed signal, so we must give
-                            # focus back to shell
+        if help_enabled:
+            doc = {
+                'name': text,
+                'ignore_unknown': False,
+            }
+            self.sig_help_requested.emit(doc)
 
         # Show calltip
         if call and getattr(self, 'calltips', None):
@@ -1508,7 +1495,7 @@ class SaveHistoryMixin(object):
     SEPARATOR = None
     HISTORY_FILENAMES = []
 
-    append_to_history = None
+    sig_append_to_history_requested = None
 
     def __init__(self, history_filename=''):
         self.history_filename = history_filename
@@ -1546,8 +1533,9 @@ class SaveHistoryMixin(object):
             encoding.write(text, self.history_filename, mode='ab')
         except EnvironmentError:
             pass
-        if self.append_to_history is not None:
-            self.append_to_history.emit(self.history_filename, text)
+        if self.sig_append_to_history_requested is not None:
+            self.sig_append_to_history_requested.emit(
+                self.history_filename, text)
 
 
 class BrowseHistory(object):
